;   MC68HCS12 SCI I/O Example
;   Source File:  serial_io.ASM
;   This file gives subroutines for SCI input
;   and output.
; Assembler routines:
;   init_sci, initsci                Initialize SCI0 9600,8,N,1
;   put_sci_0, putchar               Put char to SCI 0
;   get_sci_0, getchar               Get char from SCI 0
;   sci_0_char_ready, scichar_ready  Check for char in SCI receiver
; C Routines
;   void initsci( void);             Initialize SCI0 9600,8,N,1
;   void putchar( char );            Put char to SCI 0
;   char getchar( void );            Get char from SCI 0
;   bool scichar_ready(void);        Check for char in SCI receiver

; 
; SCI port definitions
        INCLUDE "sci.inc"
;
CR:     SET    $0d                ; Carriage return char
        XDEF   init_sci_0, initsci; Initialize sci 9600, 8, N 1
        XDEF   sci_0_char_ready   ; See if character ready
        XDEF   scichar_ready
        XDEF   get_sci_0, getchar ; Get char
        XDEF   put_sci_0, putchar ; Put a char

;***************************************
; Subroutine init_sci_0
; void initsci( void );
; Initialize SCI to 1 start, 8 data and 1 stop
; bit, no parity and 9600 Baud.
; Inputs:   None
; Outputs:  None
; Reg Mod:  CCR
init_sci_0:
initsci:
		  pshd		; Save D reg
; Set 1 start, 8 data and 1 stop bit
		  bclr	SCICR1,MODE  
; Disable parity for no parity
      bclr  SCICR1,PE
; Enable transmitter and receiver
		  bset	SCICR2,TE|RE    
; Set Baud rate
      ldd  #B115200 ; #B9600*3  ; RCM was B9600, then *6, changed due to bus clock speed change RCM
		  std 	SCIBDH 
		  nop	
		  std   SCIBDH
		  puld 		; Restore D
		  rts
;***************************************
; Subroutine put_sci_0
; void putsci( char );
; Send SCI data
; Inputs:   B register = data to send
; Outputs:  None
; Reg Mod:  CCR
put_sci_0:
putchar:
; Wait until the transmit data reg is empty
spin_put_0:	
      brclr	SCISR1,TDRE,spin_put_0
; Output the data and reset TDRE
		  stab 	SCIDRL
		  rts
;***************************************
; Subroutine sci_0_char_ready
; Check the RDRF flag
; If a character is ready, returns with C=1
; the character in the B register, and the
; status information in the A register.
; Otherwise, C=0 and the A and B regs 0 
; Inputs:   None
; Outputs:  B = character, Carry bit T or F
;           A = status information
; Reg Mod:  A,B CCR
sci_0_char_ready:
scichar_ready:
		  clc 		      ; Clear carry
		  clra          ; Clear A
		  clrb          ; Clear B
; IF RDRF is set
		  brclr	SCISR1,RDRF,exit_ready_0
; THEN the character is there
		  ldab	SCIDRL	; Get the data
		  ldaa 	SCISR1	; Get the status
		  sec 		      ; Set the carry
; ENDIF
exit_ready_0:	
      rts
;***************************************
; Subroutine sci_input_0
; char getchar( void );
; Get a character from SCI
; Waits until character has been received
; Inputs:   None
; Outputs:  B = character
; Reg Mod:  B, CCR
get_sci_0:
getchar:
      psha
; IF a char is there waiting
freds_goof:
      jsr   sci_0_char_ready
      bcc   freds_goof   ; ELSE wait
; THEN get it and return
      ldab  SCIDRL
      pula
      rts
